/**
 * Object that represents a newsfeed entry
 * @author: Daniel Puscher
 */

function NewsfeedEntry(el) {
    var $ = jQuery;
    var self = this;

    var privacy = PRIVACY.UNKNOWN;
    var privacyRead = false;
    var icon = "";
    var element = el;

    // check if the element has an id, if not add a custom one
    if (!$(el).attr("id")) {
        $(el).attr("id", "nf_unit_" + randomString(10) + "_c");
    }
    this.id = $(el).attr("id");


    var pr = privacyReader.getInstance();
    var pv = privacyVisualizer.getInstance();


    /**
     * This initializes the timeline entry object:
     * Read privacy and type
     */
    function init() {
        readPrivacy();
    }

    /**
     * Will be thrown when an error occurs
     * @param message
     */
    function NewsfeedEntryException(message) {
        this.message = message;
        this.name = "NewsfeedEntryException";
        this.toString = function () {
            return this.name + ": " + this.message;
        }
    }

    /**
     * Returns the privacy
     * @param callback
     */
    this.getPrivacy = function (callback) {
        if (!privacyRead) {
            readPrivacy(callback);
        } else {
            callback(privacy);
        }
    };

    /**
     * Colorize comment-fields depending on the visibility of the post:
     */
    function colorizeCommentFields() {
        $(element).find("textarea[name^=add_comment_text]:not([fpw])").each(function (k, v) {
            $(v).attr("fpw", "processed");

            self.getPrivacy(function (setting) {
                if (setting >= 0 && setting <= 3) {
                    pv.colorize(v.parentNode.parentNode, setting);
                }
            });
        });
    }

    /**
     * Colorize the like links (add a colored bullet next to it)
     */
    function colorizeLikeLinks() {
        $(element).find(".UFILikeLink:not(\".accessible_elem\"):not(\"[fpw]\")").each(function (k, v) {
            $(v).attr("fpw", "processed");

            var likeIndicator = $("<span fpw=\"likelink\">&#9673;</span>&nbsp;");
            likeIndicator.css("margin-right", "3px").css("cursor", "default");
            $(v).parent().prepend(likeIndicator);

            self.getPrivacy(function (setting) {
                if (setting >= 0 && setting <= 3) {
                    pv.colorizeText(likeIndicator, setting);

                    // Set a tooltip for the colored icon:
                    var tooltip = null;
                    switch(setting){
                        case 0:
                            tooltip = getLocString("newsfeed_like_public");
                            break;
                        case 1:
                            tooltip = getLocString("newsfeed_like_friends");
                            break;
                        case 3:
                            tooltip = getLocString("newsfeed_like_custom");
                    }

                    // Only colorize if a string for the tooltip was loaded:
                    if(!!tooltip){
                        likeIndicator.attr({
                            "data-hover": "tooltip",
                            "data-tooltip-alignh": "center",
                            "aria-label": tooltip
                        });
                    }
                }
            });
        });
    }

    /**
     * Reset the comment fields: remove color
     */
    function resetCommentFields() {
        $(element).find("textarea[name^=add_comment_text][fpw]").each(function (k, v) {
            $(v).removeAttr("fpw");
            pv.resetElement(v.parentNode.parentNode);
        });
    }

    /**
     * Reset the like links: Remove info-dots
     */
    function resetLikeLinks(){
        $(el).find("[fpw=\"likelink\"]").remove();
        $(el).find("[fpw]").removeAttr("fpw");
    }

    /**
     * Read privacy settings of the element
     * @param {Function}    [callback]    Callback function, privacy value will be passed as attribute
     */
    function readPrivacy(callback) {
        pr.read(element, function (privacy_value, icon_value) {
            // save the read privacy setting and icon (base64 string)
            privacy = privacy_value;
            icon = icon_value;

            privacyRead = true;

            if (typeof callback === "function") {
                callback(privacy_value);
            }
        });
    }

    /**
     * Colorize the like and comment-input fields
     */
    this.colorize = function () {
        colorizeCommentFields();
        colorizeLikeLinks();
    };

    /**
     * TODO: Delete this, only used for debugging!
     * @returns {{privacy: (PRIVACY.UNKNOWN|*), icon: string, type: number}}
     */
    this.getInfo = function () {
        return {
            id: this.id,
            element: element,
            privacy: privacy,
            icon: icon,
            type: type
        }
    };

    /**
     * Reset the element
     */
    this.reset = function () {
        privacy = PRIVACY.UNKNOWN;
        privacyRead = false;
        icon = "";

        resetCommentFields();
        resetLikeLinks();
    };

    /**
     * Check if the element is still active in DOM
     * @returns {boolean}
     */
    this.active = function () {
        return $(element).closest('body').length > 0;
    };

    /**
     * Check if the element is in the viewport
     * @returns {boolean}
     */
    this.inView = function () {
        return $(element).isOnScreen();
    };

    // init the instance of the object!
    init();
}