/**
 * Default preferences
 * @type {{border: Array, background: Array, useImages: boolean, animate: boolean}}
 */
var prefs = {
    border: ["#7DFF6E", "#FFB25E", "#FF6363", "#8FAEFF"],
    borderImages: ["#333333", "#333333", "#333333", "#333333"],
    background: ["#BAFFB2", "#FFDAB2", "#FFA3A3", "#CCDAFF"],
    backgroundImages: [sprintf("url(%s)", getFileUrl("img/bg_noColor_0.gif")),
        sprintf("url(%s)", getFileUrl("img/bg_noColor_1.gif")),
        sprintf("url(%s)", getFileUrl("img/bg_noColor_2.gif")),
        sprintf("url(%s)", getFileUrl("img/bg_noColor_3.gif"))],
    useImages: false,
    animate: true
};

/**
 * Enumeration: Privacy settings
 * @type {{UNKNOWN: number, PUBLIC: number, FRIENDS: number, ONLYME: number, CUSTOM: number}}
 */
var PRIVACY = {
    UNKNOWN : -1,
    PUBLIC  : 0,
    FRIENDS : 1,
    ONLYME  : 2,
    CUSTOM  : 3
};

/**
 * Runtime properties
 * @type {{userid: null, scrollObserver: boolean, clickObserver: boolean}}
 */
var runtime = {
    userid: null,
    scrollObserver: false,
    clickObserver: false
};

/**
 * Page indicators (used with jQuery to check which page is displayed)
 * @type {{newsfeed: string, timeline: string}}
 */
var pageIndicators = {
    newsfeed: "#stream_pagelet",
    timeline: "#pagelet_timeline_recent",
};

/**
 * Enumeration: Timeline entry types
 * @type {{UNKNOWN: number, CHANGEABLE: number, NOCHANGE: number, IGNORE: number, TAGGED: number, WALLPOST: number, FRIENDLIST: number, LIKES: number}}
 */
var ENTRYTYPES = {
    UNKNOWN : -1,
    CHANGEABLE : 0,
    NOCHANGE : 1,
    IGNORE: 2,
    TAGGED: 3,
    WALLPOST: 4,
    FRIENDLIST: 5,
    LIKES: 6
};

/**
 * Enumeration: Page types
 * @type {{NEWSFEED: string, ABOUT: string, TIMELINE: string, FRIENDS: string, LIKES: string, FAVORITES: string, ALBUMS: string, OTHERAPPS: string, RECENTACTIVITY: string, PHOTOOVERLAY: string, SINGLETIMELINE: string, SINGLEPHOTO: string, ACTIVITYLOG: string, LIKESUGGESTION: string, SINGLELIKE: string, LIST: string}}
 */
var pageTypes = {
    NEWSFEED: "newsfeed",
    TIMELINE: "timeline",
};

/**
 * Update the default settings of the visualizer with user-defined settings (async)
 */
function setPrivacyVisualizerPrefs() {
    jQuery.when(
        getPref("useImages", function (r) {
            prefs.useImages = r
        }),
        getPref("background", function (r) {
            prefs.background = r
        }),
        getPref("border", function (r) {
            prefs.border = r
        }),
        getPref("animate", function (r) {
            prefs.animate = r
        })
    ).then(function () {
        var pv = privacyVisualizer.getInstance();

        pv.updateSettings({
            useImages: prefs.useImages,
            background: prefs.background,
            border: prefs.border,
            animate: prefs.animate
        });
    });
}



function addInfoDiv() {
    try {
        var bg = (!!prefs.useImages) ? prefs.backgroundImages : prefs.background;
        var bo = (!!prefs.useImages) ? prefs.borderImages : prefs.border;
        var linePattern = '<tr><td><div style="background:%s center; border:1px solid %s;"></div></td><td><span>%s</span></td></tr>';

        if (jQuery("#fpwQuickinfo").length === 0) {
            var infoDiv = jQuery('<div/>');
            infoDiv.attr("id", "fpwQuickinfo");
            infoDiv.css("backgroundImage", sprintf("url('%s')", getFileUrl("img/quickinfo_bg.png")));

            infoDiv.html('<div><table>' +
                sprintf(linePattern, bg[0], bo[0], getLocString("public")) +
                sprintf(linePattern, bg[1], bo[1], getLocString("friends")) +
                sprintf(linePattern, bg[2], bo[2], getLocString("onlyme")) +
                sprintf(linePattern, bg[3], bo[3], getLocString("custom")) +
                '</table></div>');

            infoDiv.hover(function (over) {
                //if(!!prefs.animate){
                infoDiv.stop(true, false).animate({
                    left: -1
                }, 500);
                //} else {
                //    infoDiv.css("left", -1);
                //}
            }, function (out) {
//                if(!!prefs.animate){
                infoDiv.stop(true, false).animate({
                    left: -220
                }, 500);
//                } else {
//                    infoDiv.css("left", -220);
//                }
            });

            addElement(document.body, infoDiv);
        }
    }
    catch (e) {
        logError(e);
    }
}



function getLoggedInUser() {
    var cookie_obj = str2obj(document.cookie.toString());
    if (cookie_obj.hasOwnProperty("c_user")) {
        return cookie_obj["c_user"];
    }
    return null;
}




/**
 * Refresh all elements on the current page
 */
function refreshPage(){
    timelineProcessor.refreshAll();
    newsfeedProcessor.refreshAll();
}




/**
 * Clear the page that is displayed at the moment
 */
function clearPage() {
    // Reset all elements
    timelineProcessor.resetAll();
    newsfeedProcessor.resetAll();

    // Remove quickinfo
    removeElementsById("fpwQuickinfo");
    removeElementsById("fpwChangeOverlay");
}





function processPage() {
    runtime.userid = getLoggedInUser();
    if (!!runtime.userid) {
        /*getPref("showHelp", function (showHelp) {
            if (showHelp === true) {
                fpwHelp.init();
            }
            else { */
                addInfoDiv();

                if(showingUsersProfile()){
                    timelineProcessor.go();
                }

                newsfeedProcessor.go();
/*            }
        });*/
    } else {
        clearPage();
    }
}



function addObserver() {
    if(!runtime.scrollObserver){
        jQuery(window).on("scrollstop", function(){
            console.log("scrollstop");
            checkIfActive();
        });
        runtime.scrollObserver = true;
    }

    if(!runtime.clickObserver){
        jQuery(document).click(function(e) {
            setTimeout(function(){
                console.log('clicked');
                checkIfActive();
            }, 500);
        });
        runtime.clickObserver = true;
    }
}

function showingUsersProfile() {
    if (jQuery("#pagelet_timeline_main_column").length > 0) {
        var data = jQuery("#pagelet_timeline_main_column").attr("data-gt");
        if (data) {
            if (JSON.parse(data)["profile_owner"] === runtime.userid) {
                return true;
            }
        }
    }
    else if (jQuery(".profileThumb").find("img").length > 0) {
        if (jQuery(".profileThumb").find("img").attr("src").split("/").pop().split("_")[1] === runtime.userid) {
            return true;
        }
    }

    //Single Photo Page:
    else if (jQuery("#fbPhotoPageAuthorPic").length > 0 && jQuery("#fbPhotoPageAuthorPic").html().search(runtime.userid) !== -1) {
        return true;
    }

    //Single Timeline Entry
    else if (jQuery(".permalink_stream").length > 0 && jQuery(".permalink_stream .profilePic").attr("src").search(runtime.userid) !== -1) {
        return true;
    }

    //Calendar view
    else if (jQuery("body.fbCalendar a[href*='create/dialog']").length > 0) {
        return true;
    }

    return false;
}



function processCreateEvent() {
    try {
        var dialog = jQuery(pageIndicators.createEvent).closest("div[role=dialog]:not([fpw])");

        if (dialog.length > 0) {
            dialog.attr("fpw", "processed");

            var markAsPublicEvent = function () {
                var val = dialog.find("select[name*='audience']").val();
                if (val === "80" && dialog.find(".fpw_message").length == 0) {
                    dialog.css("border", "8px solid rgb(255, 0, 0)");

                    var msg = jQuery('<div/>').html('<div class="fpw_message">' +
                        '<h1>' + getLocString("createPublicEvent1") + '</h1>' +
                        '<h2>' + getLocString("createPublicEvent2") + '</h2>' +
                        '<p>' + getLocString("createPublicEvent3") + '</p></div>').contents();
                    dialog.children().first().before(msg);
                }
                else if (val !== "80") {
                    dialog.css("border", "");
                    dialog.find(".fpw_message").remove();
                }
            };

            if (dialog.find(".audienceSelector li").length > 0) {
                dialog.find(".audienceSelector li a").click(function () {
                    window.setTimeout(function(){
                        markAsPublicEvent();
                    }, 100);
                });
                markAsPublicEvent();
            }
        }
    } catch (e) {
    }
}

// Extension is started in "content.js", when storageObject is emitted
