var fpwHelp = (function ($) {

    /**
     * Add the css font to the document
     */
    function addCssFont(){
        var s = $('<link/>').attr({
            href: "//fonts.googleapis.com/css?family=Indie+Flower",
            rel: "stylesheet",
            type: "text/css"
        });

        $("head").append(s);
    }

    function init() {
        if ($(pageIndicators.timeline).length > 0) {

            if ($("#fpwHelpOverlay").length === 0) {
                runtime.helpStage = 0;

                addCssFont();
                addOverlay();
                refresh();

                $(window).resize(function () {
                    $("[fpwHelp]").css("left", $(window).width() / 2 - 500);
                });
            }
        }
        else {
            location.href = "http://www.facebook.com/profile.php";
        }
    }

    function refresh() {
        var pr = privacyReader.getInstance();
        var pv = privacyVisualizer.getInstance();

        if (runtime.helpStage <= 6) {
            changeText(getText());
            $("#nextHelp").click(next);
            $("#closeHelp").click(close);

            if (runtime.helpStage === 2) {
                var tl_entry = $(".userContentWrapper:visible").parent();
                if (tl_entry.length > 0) {
                    var el = tl_entry[0];
                    var el_width = $(el).width();

                    $(el).attr("fpwHelp", "zIndexChanged").css({
                        "zIndex": 5000,
                        "position": "fixed",
                        "left": $(window).width() / 2 - 500,
                        "top": "50%",
                        "width": el_width
                    });

                    pr.read(el, function (p) {
                        if (p >= 0 && p <= 3) {
                            var privacyString =
                                (p === 0) ? getLocString("public") : (
                                    (p === 1) ? getLocString("friends") : (
                                        (p === 2) ? getLocString("onlyme") :
                                            getLocString("custom")));

                            $("#helpReplaceWithPrivacy").text(privacyString);
                            pv.colorize(el, p);
                        }
                    });
                }
            }
            else if (runtime.helpStage === 3) {
                var el = $("[fpwHelp]")[0];
                if (el) {
                    var img = $("<img/>").attr({
                        id: "helpColorWheel",
                        src: getFileUrl("img/colorWheel.png")
                    }).css({
                        position: "absolute",
                        top: "-25px",
                        left: "400px",
                        zIndex: 600
                    });

                    addElement(el, img);
                }
            }
            else if (runtime.helpStage === 4) {
                $("[fpwHelp]").css({
                    "zIndex": "auto",
                    "position": "relative",
                    "left": "auto",
                    "top": "auto"
                }).removeAttr("fpwHelp");

                $("[fpw]").each(function (i, el) {
                    pv.resetElement(el);
                });

                removeElementsById("helpColorWheel");

                addInfoDiv();
            }
            else {
                removeElementsById("fpwQuickinfo");
            }
        }
        else {
            close();
        }
    }

    function addOverlay() {
        if ($("#fpwHelpOverlay").length === 0) {
            var div = document.createElement("div");
            div.id = "fpwHelpOverlay";
            div.innerHTML = '<div></div><a id="closeHelp" class="button">x</a>';

            addElement(document.body, div);
        }
    }

    function close() {
        setPref("showHelp", false);
        location.reload();
    }

    function getText() {
        var newTxt = "<p>";
        if (runtime.helpStage === 0) {
            newTxt += getLocString("helpText0");
        }
        if (runtime.helpStage === 1) {
            var bg = (prefs.useImages === true) ? prefs.backgroundImages : prefs.background;
            var bo = (prefs.useImages === true) ? prefs.borderImages : prefs.border;
            var linePattern = '<span class="sampleColor" style="background:%s center; border:1px solid %s;" />%s<br>';


            newTxt += getLocString("helpText1");
            newTxt += '</p><p style="font-size: 90%">';
            newTxt += sprintf(linePattern, bg[0], bo[0], getLocString("public"));
            newTxt += sprintf(linePattern, bg[1], bo[1], getLocString("friends"));
            newTxt += sprintf(linePattern, bg[2], bo[2], getLocString("onlyme"));
            newTxt += sprintf(linePattern, bg[3], bo[3], getLocString("custom"));
        }
        else if (runtime.helpStage === 2) {
            newTxt += getLocString("helpText2");
            newTxt += " <span id=\"helpReplaceWithPrivacy\"></span>";
        }
        else if (runtime.helpStage === 3) {
            newTxt += getLocString("helpText3");
            newTxt += '<br><img src="' + getFileUrl("img/redArrow2.png") + '" style="width: 230px; position: absolute; top: 50%; left: 600px; margin-right: -40px; margin-top: -110px;">';
        }
        else if (runtime.helpStage === 4) {
            newTxt += getLocString("helpText4");
            newTxt += '<br><img src="' + getFileUrl("img/redArrow.png") + '" style="width: 250px; position: absolute; top: 50%; margin-left: -125px; margin-top: 40px;">';
        }
        else if (runtime.helpStage === 5) {
            newTxt += getLocString("helpText5");
            newTxt += '<br><img src="' + getFileUrl("img/redArrow3.png") + '" style="width: 175px; position: fixed; top: 30px; right: 100px;">';
        }
        else if (runtime.helpStage === 6) {
            newTxt += getLocString("helpTextEnd1");
            newTxt += "</p><p>";
            newTxt += getLocString("helpTextEnd2");
        }

        newTxt += '</p><a class="button" id="nextHelp">' + getLocString("helpNext") + '</a>';

        return newTxt;
    }

    function changeText(txt) {
        if ($("#fpwHelpOverlay").find("> div").length > 0) {
            $("#fpwHelpOverlay").find("> div").html(txt);
        }
        else {
            addOverlay();
        }
    }

    function next() {
        runtime.helpStage++;

        refresh();
    }

    return {
        init: init
    }

})(jQuery);