/**
 * Add an element as child of another one
 * @param {HTMLElement} parent  Parent node
 * @param {HTMLElement} child   Child node that is added to the dom
 */
function addElement(parent, child) {
    if (prefs.animate === true) {
        jQuery(child).hide();
        jQuery(parent).append(child);
        jQuery(child).fadeIn();
    }
    else {
        jQuery(parent).append(child);
    }
}

/**
 * Log the given error to the console
 * @param {Error}   e   The error to process
 */
function logError(e){
    console.error(e.name + ": " + e.message);
}

/**
 * Remove elements from the DOM.
 * If animation is turned on it will be faded out and removed afterwards
 * @param {HTMLElement} el  Element that will be removed from DOM
 */
function removeElements(el) {
    if (prefs.animate === true) {
        jQuery(el).fadeOut(function () {
            jQuery(this).remove();
        });
    }
    else {
        jQuery(el).remove();
    }
}

/**
 * Remove an element with a given ID.
 * @param {String}  id  ID of the element that will be deleted
 */
function removeElementsById(id) {
    removeElements(jQuery(sprintf("#%s", id)));
}

/**
 * Convert a string into an object, with the given format:
 * key=value; key=value; ...
 * @param {String}  s   Input string that will be converted to an object
 * @param {String}  [splitEl]   String that is used to split the elements
 * @param {String}  [splitKV]   String that is used to split key and value
 * @returns {{}}
 */
function str2obj(s, splitEl, splitKV) {
    splitEl = (splitEl) ? splitEl : "; ";
    splitKV = (splitKV) ? splitKV : "=";

    var obj = {};
    var properties = s.split(splitEl);
    _.each(properties, function (v) {
        obj[v.split(splitKV)[0].trim()] = v.split(splitKV)[1].trim();
    });
    return obj;
}

/**
 * Generates a random string of the given length
 * @param {Number}  [length]    Length of the random String (default: 20)
 * @returns {String}    A random string of given length
 */
function randomString(length) {
    length = (length || 10);

    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    for (var i = 0; i < length; i++){
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    }

    return text;
}