var tmpStorage;

function getPref(name, callback) {
    if (!!tmpStorage[name]) {
        callback(tmpStorage[name]);
    }
    else {
        callback(undefined);
    }
}

function setPref(name, value, index) {
    if (typeof index === "number") {
        getPref(name, function (pref) {
            //If an index ist set, change only the corresponding value
            if (pref && index >= 0 && index < pref.length) {
                pref[index] = value;
                setPref(name, pref);
            }
        });
    }
    else {
        tmpStorage[name] = value;
        self.port.emit("setPref", name, value);
    }
}

function getCache(user, callback) {
    getPref("cache", function (v) {
        var cache = v[user];
        callback((cache) ? cache : {});
    });
}

function getFilled(user, callback) {
    getPref("filled", function (v) {
        var filled = v[user];
        callback((filled) ? filled : {});
    });
}

function setObject(type, user, name, value, subId) {
    saveToInitial(user, name, value, subId);

    getPref(type, function (pref) {
        if (!pref[user]) {
            pref[user] = {};
        }

        if (typeof subId !== "undefined") {
	        if (pref[user][name] == undefined){
	            pref[user][name] = Array();
	        }
	        else if(subId !== 0 && typeof pref[user][name] !== "object"){
	            pref[user][name] = [pref[user][name]];
	        }
	        pref[user][name][subId] = value;
	    }
	    else {
	        pref[user][name] = value;
	    }

        setPref(type, pref);
    });
}

function saveToInitial(user, name, value, subId) {
    getPref("initial", function (initial) {
        if (!initial[user]) {
            initial[user] = {};
        }

        if (typeof subId !== "undefined") {
            if (initial[user][name] == undefined) {
                initial[user][name] = [];
                initial[user][name][subId] = value;
            }
            else if (initial[user][name][subId] == undefined) {
                initial[user][name][subId] = value;
            }
        }
        else {
            if (!initial[user][name]) {
                initial[user][name] = value;
            }
        }
        setPref("initial", initial);
    });
}

function addToCache(user, name, value, subId) {
    if (typeof subId !== "undefined") {
        setObject("cache", user, name, value, subId);
    }
    else {
        setObject("cache", user, name, value);
    }
}

function setFilled(user, name, value, subId) {
    if (typeof subId !== "undefined") {
        setObject("filled", user, name, value, subId);
    }
    else {
        setObject("filled", user, name, value);
    }
}

function addToStats(user, sub, prop, type, subId) {
    getPref("stats", function (currentStats) {
        if (currentStats[user] == undefined)
            currentStats[user] = {};

        if (type === "append") {
            if (currentStats[user][sub] == undefined)
                currentStats[user][sub] = [];
            currentStats[user][sub].push(prop);
        }
        else if (type === "object") {
            if (currentStats[user][sub] == undefined)
                currentStats[user][sub] = {};
            currentStats[user][sub][subId] = prop;
        }
        else {
            currentStats[user][sub] = prop;
        }

        setPref("stats", currentStats);
    });
}

function resetPrefs() {
    setPref("active", true);
    setPref("background", ["#baffb2", "#ffdab2", "#ffa3a3", "#ccdaff"]);
    setPref("border", ["#7aff6b", "#ffb86b", "#ff5c5c", "#85a6ff"]);
    setPref("useImages", false);
    setPref("animate", true);
    setPref("cache", {});
    setPref("filled", {});
}