/**
 * PrivacyChanger lets you change the privacy settings of an element
 * @author Daniel Puscher
 */
var privacyChanger = (function () {
    //Stores a reference to the instance (singleton-pattern)
    var instance;

    /**
     * Creates an instance of the privacy visualizer
     * @returns {{change: change}}
     */
    function init() {

        /**
         * Exception that is thrown when an error happened during reading privacy
         * @param message   Error-message
         * @param [page]    page where the error occured
         * @param [id]      Id of the element that caused the error
         * @constructor
         */
        function PrivacyChangerException(message, page, id) {
            if (!page) {
                page = "";
            }
            if (!id) {
                id = "";
            }
            this.message = message;
            this.name = "PrivacyChangerException";
            this.page = page;
            this.id = id;

            this.toString = function(){
                return this.name + ": " + this.message + " (Page: " + this.page + ", ID: " + this.id + ")";
            }
        }

        /**
         * Waits till the custom settings dialog is closed
         * @param callback  Function that is called after the dialog was closed
         */
        function waitForCustomDialog(callback) {
            var interval = setInterval(function () {
                if (jQuery("form[action='/ajax/privacy/custom_save/']:visible, form[action='/ajax/privacy/custom_get/']:visible").length == 0) {
                    clearInterval(interval);
                    callback();
                }
            }, 500);
        }

        /**
         * Opens the edit-menu
         * @returns {*}
         */
        function openEdit() {
            var dfd2 = jQuery.Deferred();

            var searchString = "[href*='edit_dialog.php']:visible, [href*='edit_privacy.php']:visible, [ajaxify*='album/edit/dialog']:visible";
            jQuery(searchString).fclick();

            var interval = setInterval(function () {
                if (jQuery("[role*='dialog']:visible .uiSelectorButton").length > 0) {
                    clearInterval(interval);
                    dfd2.resolve();
                }
                else {
                    jQuery(searchString).fclick();
                }
            }, 500);

            return dfd2.promise();
        }

        function openMenu(el) {
            var dfd = jQuery.Deferred();
            jQuery(el).find(".uiPopoverButton:first").fclick();

            var interval = setInterval(function () {
                if (jQuery("[href*='edit_dialog.php']:visible, [href*='edit_privacy.php']:visible").length > 0) {
                    clearInterval(interval);
                    dfd.resolve();
                }
                else {
                    jQuery(el).find(".uiPopoverButton:first").fclick();
                }
            }, 500);

            return dfd.promise();
        }

        function openEditAbout(el, sub) {
            var dfd = jQuery.Deferred();
            jQuery("#fbTimelineEditProfileDialog").remove();

            var interval = setInterval(function () {
                if (jQuery(el).find(".uiSelectorButton").length > 0) {
                    clearInterval(interval);
                    dfd.resolve();
                }
                else if (jQuery("#fbTimelineEditProfileDialog:visible").length > 0) {
                    jQuery("#fbTimelineEditProfileDialog").hide();
                    clearInterval(interval);
                    dfd.resolve();
                }
                else {
                    if (jQuery(el).find(".profileEditButton:visible").length > 0) {
                        jQuery(el).find(".profileEditButton:visible").first().fclick();
                    }
                    else if (el.hasAttribute("fpw")) {
                        if (typeof(sub) != "undefined") {
                            jQuery(jQuery(el).find("a.editForm")[sub]).fclick();
                        } else {
                            jQuery(el).find("a.editForm").fclick();
                        }
                    }
                    else {
                        if (typeof(sub) != "undefined") {
                            jQuery(jQuery(el).find("tr[fpw] a.editForm[href*='ajax/timeline/edit_profile']," +
                                "div[fpw] a.editForm[href*='ajax/timeline/edit_profile']")[sub]).fclick();
                        } else {
                            jQuery(el).find("tr[fpw] a.editForm[href*='ajax/timeline/edit_profile']," +
                                "div[fpw] a.editForm[href*='ajax/timeline/edit_profile']").fclick();
                        }
                    }
                }
            }, 500);

            return dfd.promise();
        }

        function openSkills(el) {
            var dfd = jQuery.Deferred();

            var interval = setInterval(function () {
                if (jQuery(el).find(".uiSelectorButton").length > 0) {
                    clearInterval(interval);
                    dfd.resolve();
                }
                else {
                    jQuery(el).find("form[action*='edit_skills']").fsubmit();
                }
            }, 500);

            return dfd.promise();
        }

        function usePrivacyMenu(menu, new_setting, el, callback) {
            jQuery(menu).css("opacity", 0); //hide privacy menu

            // convert the privacy setting to the value facebook is using
            var fb_value = (new_setting == PRIVACY.PUBLIC) ? 80 : (
                                (new_setting == PRIVACY.FRIENDS) ? 40 : (
                                    (new_setting === PRIVACY.ONLYME) ? 10 : -1
                                )
                            );

            // Get the button from the menu that corresponds to the new setting
            var new_setting_button = (fb_value < 0) ?
                jQuery(menu).find(".customPrivacyInputs") :
                jQuery(menu).find("[ajaxify*='%22value%22%3A" + fb_value + "%7D']:visible");


            if (new_setting_button.length > 0) {
                new_setting_button.fclick(); // set the new privacy setting
                jQuery(menu).css("opacity", 1); //undo hiding of privacy menu

                // wait for the custom dialog
                waitForCustomDialog(function () {
                    //Remove all open windows that were used when changing privacy settings
                    jQuery("form[action*='done=1'] input[type=submit]").fclick();

                    removeElements(jQuery("[role='dialog']:visible"));

                    callback(true);
                });
            }
            else {
                var button = jQuery(menu).find("a.uiSelectorButton.uiButton.uiButtonSuppressed");
                if (button.length > 0) {
                    button.fclick();
                    window.setTimeout(function () {
                        usePrivacyMenu(menu, new_setting, el, callback)
                    }, 500);
                }
            }
        }

        function usePrivacyMenuAbout(menu, value, el, callback) {
            var fb_value = (value == 0) ? 80 : ((value == 1) ? 40 : ((value === 2) ? 10 : -1));

            if (fb_value < 0) {
                // New setting == Custom
                var customLink = jQuery(menu).find(".customPrivacyInputs");
                if (customLink.length > 0) {
                    customLink.fclick();
                    jQuery.when(waitForCustomDialog()).then(function () {
                        jQuery(menu).parent().parent().find("select[name*='audience']").val("111"); //Custom
                        jQuery("#fbTimelineEditProfileDialog").find("button[type=submit]").fclick();
                        jQuery("form[ajaxify*='/ajax/timeline/edit_profile/'] button[type=submit]").fclick();
                        jQuery("form[ajaxify^='/profile/async/edit'] button[type=submit]").fclick();
                    });
                }
                else {
                    var button = jQuery(menu).find("div.audienceSelector a.uiSelectorButton");
                    if (button.length > 0) {
                        button.fclick();
                        window.setTimeout(function () {
                            usePrivacyMenuAbout(menu, value, el, callback)
                        }, 500);
                    }
                }
            }
            else {
                // New Setting == Public, Friends, Only me
                jQuery(menu).parent().parent().find("select[name*='audience']").val(fb_value.toString());
                jQuery("#fbTimelineEditProfileDialog").find("button[type=submit]").fclick();
                jQuery("form[ajaxify*='/ajax/timeline/edit_profile/'] button[type=submit]").fclick();
                jQuery("form[ajaxify^='/profile/async/edit'] button[type=submit]").fclick();
                checkIfActive();
            }
        }

        function change_privacy_inline(id, newSetting, callback) {
            var el = jQuery(sprintf("#%s", id));

            if (el.length > 0) {
                el = el[0];
                if (!jQuery(el).is('[fpw]')) {
                    var new_els = jQuery(el).find("[fpw]");
                    if (new_els.length > 0) {
                        el = new_els[0];
                    }
                    else {
                        throw new PrivacyChangerException("No element with 'fpw' attribute found!", "inline", id);
                    }
                }

                var menu = jQuery(el).find(".audienceSelector:not('.timelineRecentActivityHideSelector')");
                if (menu.length > 0) {
                    usePrivacyMenu(menu[0], newSetting, el, callback);
                } else {
                    throw new PrivacyChangerException("No privacy menu found", "inline", id);
                }
            } else {
                throw new PrivacyChangerException("No element with this id found", "inline", id);
            }
        }

        function change_privacy_friends(counter, newSetting) {
            var el = jQuery(pageIndicators.friends)[0];

            jQuery.when(openMenu(el)).then(function () {
                jQuery.when(openEdit()).then(function () {
                    var buttons = jQuery("[role=dialog] .uiSelectorButton");
                    var menu = buttons[counter];
                    addToCache(runtime.userid, privacySettingIDs.friends[counter], newSetting);
                    usePrivacyMenu(menu.parentNode, newSetting, el);
                    //Make sure all fields are colored correctly
                    window.setTimeout(function () {
                        jQuery(el).find("[fpw]").removeAttr("fpw");
                        checkIfActive();
                    }, 500);
                });
            });
        }

        function change_privacy_likes(counter, newSetting, subId) {
            var el = jQuery(pageIndicators[subId])[0];

            jQuery.when(openMenu(el)).then(function () {
                jQuery.when(openEdit()).then(function () {
                    var buttons = jQuery("[role=dialog] .uiSelectorButton");
                    var menu = buttons[counter];
                    usePrivacyMenu(menu.parentNode, newSetting, el, true);
                    //Make sure all fields are colored correctly
                    window.setTimeout(function () {
                        jQuery(el).find("[fpw]").removeAttr("fpw");
                        checkIfActive();
                    }, 500);
                });
            });
        }

        function change_privacy_favorites(counter, newSetting) {
            var el = jQuery(pageIndicators.favorites)[0];

            jQuery.when(openMenu(el)).then(function () {
                jQuery.when(openEdit()).then(function () {
                    var buttons = jQuery("[role=dialog] .uiSelectorButton");
                    var menu = buttons[counter];
                    usePrivacyMenu(menu.parentNode, newSetting, el, true);
                    //Make sure all fields are colored correctly
                    window.setTimeout(function () {
                        jQuery(el).find("[fpw]").removeAttr("fpw");
                        checkIfActive();
                    }, 500);
                });
            });
        }

        function change_privacy_about_skills(newSetting) {
            var el = jQuery(pageIndicators.about).find(aboutPagelets[0])[0];

            jQuery.when(openEditAbout(el)).then(function () {
                var row = jQuery(el).find("table.fbEditProfileExperienceSection tbody:last");

                jQuery.when(openSkills(row)).then(function () {
                    var menu = jQuery(row).find(".audienceSelector")[0];
                    addToCache(runtime.userid, "Skills", newSetting);
                    usePrivacyMenu(menu.parentNode, newSetting, el);

                    //Make sure all fields are colored correctly
                    window.setTimeout(function () {
                        jQuery(el).removeAttr("fpw");
                        jQuery(el).find("[fpw]").removeAttr("fpw");
                        checkIfActive();
                    }, 1000);
                });
            });
        }

        function change_privacy_about(pagelet, newSetting, settingName) {
            var el = jQuery(pageIndicators.about).find(aboutPagelets[pagelet])[0];
            var subId;

            if (pagelet == 2 || pagelet == 3 || pagelet == 5 || pagelet == 6) {
                subId = 0;
                if (settingName && String(settingName).search(";") !== -1) {
                    subId = settingName.split(";")[1];
                    settingName = settingName.split(";")[0];
                }
                if (!settingName) settingName = 0;

                jQuery.when(openEditAbout(el, settingName)).then(function () {
                    console.log(pagelet, newSetting, settingName);
                    var menu = jQuery(el).find(".audienceSelector")[subId];
                    usePrivacyMenuAbout(menu.parentNode, newSetting, el);
                });
            }

            else {
                jQuery.when(openEditAbout(el)).then(function () {
                    //Split name and subelement-id if present
                    if (settingName.search(";") !== -1) {
                        subId = settingName.split(";")[1];
                        settingName = settingName.split(";")[0];
                    }

                    var buttonId = privacySettingIDs.aboutParsed[pagelet].indexOf(settingName);

                    var menu;
                    if (pagelet === 0) {

                        var row = row = jQuery("#fbTimelineEditProfileDialog tbody")[buttonId];
                        if (pagelet === 0) row = jQuery("table.fbEditProfileExperienceSection tbody")[buttonId];

                        //Select correct audience-selector (if more than one belongs to the current setting, subId is set
                        if (subId != undefined) {
                            menu = jQuery(row).find(".audienceSelector")[subId];
                            addToCache(runtime.userid, settingName, newSetting, subId);
                        }
                        else {
                            menu = jQuery(row).find(".audienceSelector")[0];
                            addToCache(runtime.userid, settingName, newSetting);
                        }
                    }
                    else {
                        var buttons = jQuery("#fbTimelineEditProfileDialog div.audienceSelector .uiSelectorButton");
                        if (pagelet === 0 || pagelet === 4) {
                            buttons = jQuery(el).find(".uiSelectorButton");
                        }

                        menu = buttons[buttonId];
                        addToCache(runtime.userid, settingName, newSetting);
                    }

                    if (pagelet === 0) {
                        usePrivacyMenu(menu.parentNode, newSetting, el);
                    }
                    else {
                        usePrivacyMenuAbout(menu.parentNode, newSetting, el);
                    }

                    //Make sure all fields are colored correctly
                    window.setTimeout(function () {
                        jQuery(el).removeAttr("fpw");
                        jQuery(el).find("[fpw]").removeAttr("fpw");
                        checkIfActive();
                    }, 1000);
                });
            }
        }

        /**
         * Change the privacy of an entry
         * @param page  Page type of the page the request comes from
         * @param id    Id of the html element
         * @param newSetting    The new privacy setting
         * @param oldVal
         * @param callback
         */
        function change(page, id, newSetting, oldVal, callback) {
            if (page === pageTypes.TIMELINE || page === pageTypes.OTHERAPPS || page === pageTypes.RECENTACTIVITY ||
                page === pageTypes.PHOTOOVERLAY || page === pageTypes.SINGLETIMELINE || page === pageTypes.SINGLEPHOTO ||
                page === pageTypes.ACTIVITYLOG || page === pageTypes.LIKESUGGESTION || page === pageTypes.SINGLELIKE ||
                page === pageTypes.LIST || page === pageTypes.ALBUMS) {
                change_privacy_inline(id, newSetting, callback);
            }
            else if (page === pageTypes.FRIENDS) {
                change_privacy_friends(id, newSetting, callback);
            }
            else if (page === pageTypes.LIKES) {
                change_privacy_likes(id, newSetting, callback);
            }
            else if (page === pageTypes.FAVORITES) {
                change_privacy_favorites(id, newSetting, callback);
            }
            else if (page === pageTypes.ABOUT && subId == "Skills") {
                change_privacy_about_skills(newSetting, callback);
            }
            else if (page === pageTypes.ABOUT) {
                change_privacy_about(id, newSetting, callback);
            }

            /**
             * Add to Stats!
             */
            if (typeof subId !== "undefined") {
                id = id + "-" + subId;
            }
            addToStats(runtime.userid, "change", {page: page, id: id, oldVal: oldVal, newVal: newSetting}, "append");
        }

        return {
            change: change
        };
    }

    return {
        // Get the Singleton instance if one exists
        // or create one if it doesn't
        getInstance: function () {
            if (!instance) {
                instance = init();
            }
            return instance;
        }
    };
})();
